"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RelendWallet = void 0;
const psy_american_1 = require("@mithraic-labs/psy-american");
const anchor = __importStar(require("@project-serum/anchor"));
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const axios_1 = __importDefault(require("axios"));
const core_1 = require("../core");
const custom_spl_1 = require("../utils/custom-spl");
const merkle_distributor_1 = require("../utils/merkle_distributor");
const claim_1 = require("./claim");
const constants_1 = require("./constants");
const utils_1 = require("./utils");
// const API_ENDPOINT = "https://api.solend.fi"
const API_ENDPOINT = process.env.NEXT_PUBLIC_BACKEND_URL;
class RelendWallet {
    config;
    rewards;
    provider;
    programId;
    constructor(wallet, connection, environment) {
        this.config = null;
        this.rewards = {};
        this.provider = new anchor.Provider(connection, wallet, {});
        this.programId = (0, constants_1.getProgramId)();
    }
    static async initialize(wallet, connection, environment = "production") {
        const loadedWallet = new RelendWallet(wallet, connection, environment);
        // const config = (await (
        //   await axios.get(
        //     `${API_ENDPOINT}/v1/markets/configs?scope=all&deployment=${environment}`
        //   )
        // ).data) as ConfigType;
        const config = (0, core_1.getRelendInfo)();
        loadedWallet.config = config;
        // await loadedWallet.loadRewards();
        return loadedWallet;
    }
    async loadRewards() {
        if (!this.config) {
            throw Error("Wallet must be initialized to call loadRewards.");
        }
        const anchorProgram = new anchor.Program(merkle_distributor_1.MerkleDistributorJSON, new web3_js_1.PublicKey(merkle_distributor_1.MERKLE_PROGRAM_ID), this.provider);
        const psyOptionsProgram = new anchor.Program(psy_american_1.PsyAmericanIdl, new web3_js_1.PublicKey("R2y9ip6mxmWUj4pt54jP2hz2dgvMozy9VTSwMWE7evs"), this.provider);
        const externalStatResponse = (await axios_1.default.get(`${API_ENDPOINT}/liquidity-mining/external-reward-stats-v2?flat=true`)).data;
        const externalScoreResponse = (await axios_1.default.get(`${API_ENDPOINT}/liquidity-mining/external-reward-score-v2?wallet=${this.provider.wallet.publicKey.toBase58()}`)).data;
        const primaryMarketSeed = this.config.find((market) => market.isPrimary).address.slice(0, 32);
        const obligationAddress = await web3_js_1.PublicKey.createWithSeed(this.provider.wallet.publicKey, primaryMarketSeed, new web3_js_1.PublicKey(this.programId));
        const claimResponse = (await axios_1.default.get(`${API_ENDPOINT}/liquidity-mining/reward-proofs?obligation=${obligationAddress}`)).data;
        const [externalStatData, externalScoreData, claimData] = await Promise.all([
            externalStatResponse,
            externalScoreResponse,
            claimResponse,
        ]);
        const optionMarkets = await Promise.all(claimData.map((d) => d.optionMarketKey ? (0, psy_american_1.getOptionByKey)(psyOptionsProgram, new web3_js_1.PublicKey(d.optionMarketKey)) : Promise.resolve(null)));
        const optionAtas = await Promise.all(optionMarkets.map((om) => om
            ? spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(om.optionMint), this.provider.wallet.publicKey, true)
            : Promise.resolve(new web3_js_1.PublicKey("nu11111111111111111111111111111111111111111"))));
        const optionBalances = await this.provider.connection.getMultipleAccountsInfo(optionAtas);
        const parsedOptionBalances = optionBalances.map((om, index) => (om ? (0, custom_spl_1.unpackAccount)(optionAtas[index], om) : null));
        const merkleDistributors = (await anchorProgram.account.merkleDistributor.fetchMultiple(claimData.map((d) => d.distributorPublicKey)));
        const claimAndBumps = await Promise.all(claimData.map(async (d) => {
            const claimAndBump = await web3_js_1.PublicKey.findProgramAddress([
                anchor.utils.bytes.utf8.encode("ClaimStatus"),
                new anchor.BN(d.index).toArrayLike(Buffer, "le", 8),
                new web3_js_1.PublicKey(d.distributorPublicKey).toBytes(),
            ], new web3_js_1.PublicKey(merkle_distributor_1.MERKLE_PROGRAM_ID));
            return claimAndBump;
        }));
        const claimStatuses = (await anchorProgram.account.claimStatus.fetchMultiple(claimAndBumps.map((candb) => candb[0])));
        const fullData = await Promise.all(claimData.map(async (d, index) => {
            const [distributorATAPublicKey, _bump] = await web3_js_1.PublicKey.findProgramAddress([
                new web3_js_1.PublicKey(d.distributorPublicKey).toBuffer(),
                spl_token_1.TOKEN_PROGRAM_ID.toBuffer(),
                merkleDistributors[index].mint.toBuffer(),
            ], spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID);
            const claimable = (await this.provider.connection.getTokenAccountBalance(distributorATAPublicKey)).value.amount !== "0";
            const om = optionMarkets[index];
            return {
                ...d,
                optionMarket: om
                    ? {
                        ...om,
                        userBalance: Number(parsedOptionBalances[index]?.amount ?? 0),
                        expired: om.expirationUnixTimestamp.toNumber() <= Math.floor(new Date().getTime()) / 1000,
                    }
                    : null,
                claimable,
                claimedAt: claimStatuses[index]?.claimedAt,
                mintAddress: merkleDistributors[index].mint,
                distributor: merkleDistributors[index],
                distributorATAPublicKey,
                claimId: claimAndBumps[index][0],
                claimStatusBump: claimAndBumps[index][1],
            };
        }));
        const mostRecentSlot = await this.provider.connection.getSlot("finalized");
        const mostRecentSlotTime = (await this.provider.connection.getBlockTime(mostRecentSlot));
        const rewards = fullData.reduce((acc, currentValue) => {
            if (!acc[currentValue.distributor.mint]) {
                acc[currentValue.distributor.mint] = [];
            }
            acc[currentValue.distributor.mint].push(currentValue);
            return acc;
        }, {});
        const externalEarningsData = externalScoreData.reduce((acc, rewardScore) => {
            const rewardStat = externalStatData.find((reward) => reward.reserveID === rewardScore.reserveID && reward.side === rewardScore.side);
            const currentScore = rewardStat
                ? (0, utils_1.estimateCurrentScore)(rewardStat, rewardScore, mostRecentSlot, mostRecentSlotTime).toNumber()
                : 0;
            return {
                ...acc,
                [rewardScore.rewardMint]: {
                    symbol: rewardScore.rewardSymbol,
                    lifetimeAmount: (acc[rewardScore.rewardMint]?.lifetimeAmount ?? 0) + Number(currentScore),
                },
            };
        }, {});
        const rewardsData = {
            ...externalEarningsData,
        };
        const rewardMetadata = (await (await axios_1.default.get(`${API_ENDPOINT}/tokens/?symbols=${Object.values(rewardsData)
            .map((rew) => rew.symbol)
            .join(",")}`)).data).results;
        this.rewards = Object.fromEntries(Object.entries(rewardsData).map(([rewardMint, earning]) => {
            const rewardData = rewards[rewardMint];
            const rewardClaims = fullData
                .filter((lot) => lot.mintAddress.toBase58() === rewardMint)
                .map((reward) => new claim_1.RelendClaim(reward, this.provider));
            const metadata = rewardMetadata.find((rew) => rew?.symbol === earning.symbol);
            return [
                rewardMint,
                {
                    ...earning,
                    ...metadata,
                    lifetimeAmount: earning.lifetimeAmount / 10 ** (36 - (metadata?.decimals ?? 0)),
                    claimedAmount: rewardData
                        ?.filter((reward) => reward.claimedAt)
                        .reduce((acc, reward) => acc + Number(reward.quantity), 0) ?? 0,
                    claimableAmount: rewardData
                        ?.filter((reward) => !reward.claimedAt && reward.claimable)
                        .reduce((acc, reward) => acc + Number(reward.quantity), 0) ?? 0,
                    rewardClaims: rewardClaims,
                },
            ];
        }));
    }
    async getClaimAllIxs() {
        const allSetupIxs = [];
        const allClaimIxs = [];
        for (const claim of Object.values(this.rewards).flatMap((reward) => reward.rewardClaims)) {
            const [setupIxs, claimIxs] = await claim.getClaimIxs();
            allSetupIxs.push(...setupIxs);
            allClaimIxs.push(...claimIxs);
        }
        return [allSetupIxs, allClaimIxs];
    }
}
exports.RelendWallet = RelendWallet;
