"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchTokensInfo = void 0;
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const custom_utl_sdk_1 = __importDefault(require("../../utils/custom-utl-sdk"));
const constants_1 = require("../constants");
const fetchTokensInfo = async (mints, connection, debug) => {
    if (debug)
        console.log("getTokensInfo");
    // ! This way is using @solflare-wallet/utl-sdk package
    /*  const defaultClient = new Client();
    const config = new UtlConfig({
      ...defaultClient.config,
      chainId: IS_MAINNET ? 101 : 102,
      connection,
      apiUrl: "",
      cdnUrl:
        "https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/src/tokens/renec.tokenlist.json",
    });
    const utl = new Client(config);
  
    const tokens: Token[] = await utl.fetchMints(
      mints.map((mint) => new PublicKey(mint))
    ); */
    // ! Custom @solflare-wallet/utl-sdk package, only use neccessary codes.
    const utl = new custom_utl_sdk_1.default();
    const tokens = await utl.fetchMints("https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/src/tokens/renec.tokenlist.json", mints.map((mint) => new web3_js_1.PublicKey(mint)));
    if (debug)
        console.log("#fetchTokensInfo - tokens: ", tokens);
    let output = Object.fromEntries(tokens
        .map((token) => token
        ? [
            token.address,
            {
                name: token.name ?? "",
                symbol: token.symbol,
                logoUri: token.logoURI,
                decimals: token.decimals ?? 0,
            },
        ]
        : [])
        .filter((x) => x.length));
    if (mints.includes(spl_token_1.NATIVE_MINT.toBase58())) {
        output = {
            ...output,
            [spl_token_1.NATIVE_MINT.toBase58()]: {
                name: "RENEC",
                symbol: "RENEC",
                logoUri: "https://easytokenbucket.s3.amazonaws.com/nsbr7a24f6ox3xh40obw8jaaxjzt",
                decimals: 9,
            },
        };
    }
    if (mints.includes("USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY")) {
        output = {
            ...output,
            USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY: {
                name: "USDT (reUSD)",
                symbol: constants_1.RE_USD,
                logoUri: "https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/assets/mainnet/4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3/logo.png",
                decimals: 9,
            },
        };
    }
    if (mints.includes("4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3")) {
        output = {
            ...output,
            "4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3": {
                name: "USDT (reUSD)",
                symbol: constants_1.RE_USD,
                logoUri: "https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/assets/mainnet/4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3/logo.png",
                decimals: 9,
            },
        };
    }
    return output;
};
exports.fetchTokensInfo = fetchTokensInfo;
