"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchWalletAssets = exports.formatWalletAssets = void 0;
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const custom_spl_1 = require("../../utils/custom-spl");
const utils_1 = require("./utils");
function formatWalletAssets(rawWalletData, metadata) {
    const { userAssociatedTokenAccounts, wRENECAddress, nativeSolBalance } = rawWalletData;
    const assets = userAssociatedTokenAccounts
        .map((parsedAccount) => {
        if (!parsedAccount)
            return null;
        const mintAddress = parsedAccount.mint.toBase58();
        const tokenMetadata = metadata[mintAddress];
        const decimals = tokenMetadata?.decimals ?? 0;
        return {
            decimals,
            symbol: tokenMetadata?.symbol === "RENEC" ? "wRENEC" : tokenMetadata?.symbol,
            address: parsedAccount.address.toBase58(),
            amount: new bignumber_js_1.default(parsedAccount.amount.toString()).shiftedBy(-decimals),
            mintAddress: tokenMetadata?.symbol === "RENEC" ? "wRENEC" : mintAddress,
        };
    })
        .filter(Boolean);
    return assets.concat([
        {
            decimals: Math.log10(web3_js_1.LAMPORTS_PER_SOL),
            symbol: "RENEC",
            address: wRENECAddress,
            amount: nativeSolBalance,
            mintAddress: spl_token_1.NATIVE_MINT.toBase58(),
        },
    ]);
}
exports.formatWalletAssets = formatWalletAssets;
async function fetchWalletAssets(uniqueAssets, publicKey, connection, debug) {
    if (debug)
        console.log("fetchWalletAssets", uniqueAssets.length);
    const userTokenAssociatedAddresses = await Promise.all(uniqueAssets.map(async (asset) => {
        const userTokenAccount = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(asset), new web3_js_1.PublicKey(publicKey), true);
        return userTokenAccount;
    }));
    const userAssociatedTokenAccounts = await (0, utils_1.getBatchMultipleAccountsInfo)(userTokenAssociatedAddresses, connection);
    const nativeSolBalance = await connection.getBalance(new web3_js_1.PublicKey(publicKey));
    const wRENECAddress = (await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, spl_token_1.NATIVE_MINT, new web3_js_1.PublicKey(publicKey), true)).toBase58();
    return {
        userAssociatedTokenAccounts: userAssociatedTokenAccounts.map((account, index) => account ? (0, custom_spl_1.unpackAccount)(userTokenAssociatedAddresses[index], account) : null),
        nativeSolBalance: new bignumber_js_1.default(nativeSolBalance).dividedBy(web3_js_1.LAMPORTS_PER_SOL),
        wRENECAddress,
    };
}
exports.fetchWalletAssets = fetchWalletAssets;
