"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const child_process_1 = require("child_process");
const custom_utl_sdk_1 = __importDefault(require("../utils/custom-utl-sdk"));
const gen_reserve_cli_1 = __importDefault(require("./gen-reserve.cli"));
const reserve_rust2json_1 = require("./utils/reserve.rust2json");
const env = "prod";
const token = "GAST";
const { reserve, command } = (0, gen_reserve_cli_1.default)(env)(token);
console.log("### reserve: ", reserve);
console.log("### command: ", command);
(0, child_process_1.exec)(command, async (error, stdout) => {
    if (error) {
        console.error(`Error executing CLI command: ${error}`);
        return;
    }
    const reserveRaw = (0, reserve_rust2json_1.reserveRust2json)(stdout);
    const tokenInfo = await getTokenInfo(reserveRaw.liquidity.mint_pubkey);
    const transformedJSON = transformToNewFormat(reserveRaw, tokenInfo);
    console.log(transformedJSON);
});
function transformToNewFormat(currentJson, tokenInfo) {
    return {
        liquidityToken: {
            coingeckoID: tokenInfo.name,
            decimals: currentJson.liquidity.mint_decimals,
            logo: tokenInfo.logoURI,
            mint: currentJson.liquidity.mint_pubkey,
            name: tokenInfo.name,
            symbol: tokenInfo.symbol,
            volume24h: 24000,
        },
        pythOracle: currentJson.liquidity.oracle_pubkey,
        switchboardOracle: currentJson.liquidity.second_oracle_pubkey,
        address: reserve,
        collateralMintAddress: currentJson.collateral.mint_pubkey,
        collateralSupplyAddress: currentJson.collateral.supply_pubkey,
        liquidityAddress: currentJson.liquidity.supply_pubkey,
        liquidityFeeReceiverAddress: currentJson.config.fee_receiver,
        userBorrowCap: currentJson.config.borrow_limit,
        userSupplyCap: currentJson.config.deposit_limit,
    };
}
async function getTokenInfo(liquidityPubkey) {
    let tokenInfo;
    if (liquidityPubkey === "USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY") {
        tokenInfo = {
            name: "USDT (reUSD)",
            symbol: "reUSD",
            address: "USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY",
            decimals: 9,
            logoURI: "https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/assets/mainnet/4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3/logo.png",
        };
    }
    else if (liquidityPubkey === "4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3") {
        tokenInfo = {
            name: "USDT (reUSD)",
            symbol: "reUSD",
            address: "USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY",
            decimals: 9,
            logoURI: "https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/assets/mainnet/4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3/logo.png",
        };
    }
    else if (liquidityPubkey === spl_token_1.NATIVE_MINT.toBase58()) {
        tokenInfo = {
            name: "RENEC",
            symbol: "RENEC",
            address: spl_token_1.NATIVE_MINT.toBase58(),
            decimals: 9,
            logoURI: "https://easytokenbucket.s3.amazonaws.com/nsbr7a24f6ox3xh40obw8jaaxjzt",
        };
    }
    else {
        const utl = new custom_utl_sdk_1.default();
        const tokens = await utl.fetchMints("https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/src/tokens/renec.tokenlist.json", [new web3_js_1.PublicKey(liquidityPubkey)]);
        tokenInfo = tokens[0];
    }
    return tokenInfo;
}
