"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseProductOracle = exports.parsePriceOracle = void 0;
const anchor_1 = require("@project-serum/anchor");
const oracleTypes_1 = require("./oracleTypes");
function parseAnchorAccount(accountName, data) {
    const discriminator = anchor_1.AccountsCoder.accountDiscriminator(accountName);
    if (discriminator.compare(data.slice(0, 8))) {
        console.error("incorrect account name during parsing");
        return null;
    }
    try {
        return oracleTypes_1.accountsCoder.decode(accountName, data);
    }
    catch (_e) {
        console.error("unknown account name during parsing");
        return null;
    }
}
class ParsablePrice {
    constructor() { }
    static parse(data) {
        if (!data) {
            return null;
        }
        try {
            return parseAnchorAccount(oracleTypes_1.AccountName.Price, data);
        }
        catch (e) {
            console.error(`error while parsing Price: ${e}`);
            return null;
        }
    }
}
class ParsableProduct {
    constructor() { }
    static parse(data) {
        if (!data) {
            return null;
        }
        try {
            return parseAnchorAccount(oracleTypes_1.AccountName.Product, data);
        }
        catch (e) {
            console.error(`error while parsing Product: ${e}`);
            return null;
        }
    }
}
const parsePriceOracle = async (connection, address) => {
    const accountInfo = await connection.getAccountInfo((0, anchor_1.translateAddress)(address));
    const accountData = accountInfo?.data;
    const value = ParsablePrice.parse(accountData);
    return value;
};
exports.parsePriceOracle = parsePriceOracle;
const parseProductOracle = async (connection, address) => {
    const accountInfo = await connection.getAccountInfo((0, anchor_1.translateAddress)(address));
    const accountData = accountInfo?.data;
    const value = ParsableProduct.parse(accountData);
    return value;
};
exports.parseProductOracle = parseProductOracle;
